<?php
/*
 * Copyright (C) 2017 PayKeeper
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * @author      PayKeeper
 * @copyright   2017 PayKeeper
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU General Public License, version 2 (GPL-2.0)
 */

namespace PayKeeper\PayKeeper\Model;

/**
 * Checkout Payment Method Model Class
 * Class Checkout
 * @package PayKeeper\PayKeeper\Model\Method
 */
class Payment extends \Magento\Payment\Model\Method\AbstractMethod
{
    const CODE = 'paykeeper_checkout';
    /**
     * Checkout Method Code
     */
    protected $_code = self::CODE;

    protected $_canOrder                    = true;
    protected $_isGateway                   = true;
    protected $_canCapture                  = true;
    protected $_canCapturePartial           = true;
    protected $_canRefund                   = true;
    protected $_canCancelInvoice            = true;
    protected $_canVoid                     = true;
    protected $_canRefundInvoicePartial     = true;
    protected $_canAuthorize                = true;
    protected $_isInitializeNeeded          = false;

    private $fiscal_cart = array(); //fz54 cart
    private $order_total = 0; //order total sum
    private $shipping_price = 0; //shipping price
    private $use_taxes = false;
    private $use_delivery = false;
    private $order_params = NULL;
    private $discounts = array();

    /**
     * Get Instance of the Magento Code Logger
     * @return \Psr\Log\LoggerInterface
     */
    protected function getLogger()
    {
        return $this->_logger;
    }

    /**
     * Checkout constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\App\Action\Context $actionContext
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory
     * @param \Magento\Payment\Helper\Data $paymentData
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Payment\Model\Method\Logger $logger
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \PayKeeper\PayKeeper\Helper\Data $moduleHelper
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Payment\Helper\Data $paymentData,
        \PayKeeper\PayKeeper\Model\PaymentConfig $paymentConfig,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\App\Action\Context $actionContext,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger  $logger,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\Order $order,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender,
        \Magento\Framework\DB\Transaction $transaction,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Magento\Quote\Model\QuoteFactory $quoteFactory,
        \PayKeeper\PayKeeper\Helper\Data $moduleHelper,
        \PayKeeper\PayKeeper\Helper\Payment $paymentHelper,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->orderRepository = $orderRepository;
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );
        $this->_actionContext = $actionContext;
        $this->_storeManager = $storeManager;
        $this->_checkoutSession = $checkoutSession;
        $this->_order = $order;
        $this->orderSender = $orderSender;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->transaction = $transaction;
        $this->invoiceService = $invoiceService;
        $this->invoiceSender = $invoiceSender;
        $this->_request = $request;
        $this->_quoteFactory = $quoteFactory;
        $this->_quoteRepository = $quoteRepository;
        $this->_moduleHelper = $moduleHelper;
        $this->_paymentHelper = $paymentHelper;
        $this->paymentConfig = $paymentConfig;
    }

    private function getOrder($orderid = Null)
    {
        if ($orderid) {
            $searchCriteria = $this->searchCriteriaBuilder->addFilter(
                "increment_id",
                $orderid,
                "eq"
                )->create();
            $orderList = $this->orderRepository->getList($searchCriteria)->getItems();
            return reset($orderList);
        }
        else return $this->_checkoutSession->getLastRealOrder();
    }

    public function getCallback()
    {
  
        // check request type
        if (!$this->_request->isPost())
            die('Wrong request type (not Post)');

        // get request variables
        $request = $this->_request->getPost();
        if (empty($request)) {
            die('Request doesn\'t contain POST elements.');
        }

        $id		    =  $request['id'];
        $sum	    =  $request['sum'];
        $clientid	=  $request['clientid'];
        $orderid	=  $request['orderid'];
        $sign	    =  $request['key'];

        // check order id
        if (empty($orderid) || strlen($orderid) > 50)
            die('Missing or invalid order ID');
  
        $secret_key = $this->_paymentHelper->getConfig("payment/paykeeper_checkout/secret_key");

        $check_sign = md5($id.$sum.$clientid.$orderid.$secret_key);
  
        // check transaction signature
        if($sign != $check_sign)
          die('Message digest incorrect');

        $order = $this->getOrder($orderid);
  
        $invoice = $this->invoiceService->prepareInvoice($order);
        $invoice->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::CAPTURE_ONLINE);
        $invoice->register();
        $invoice->save();
        $transactionSave = $this->transaction->addObject(
            $invoice
            )->addObject(
            $invoice->getOrder()
        );
        $transactionSave->save();
        $this->orderSender->send($order);
        $order->save();
  
        $hash = md5($id.$secret_key);
  
        die("OK $hash");
    }

    public function getFormFields()
    {
        $session = $this->_checkoutSession;
        $quote = $session->getQuote();
        $order = $this->getOrder();

        //set order params
        $this->paymentConfig->setOrderParams(
            //sum
            $order->getGrandTotal(),
            //clientid
            $order->getCustomerFirstName() . " " . $order->getCustomerLastName(),
            //orderid
            $order->getIncrementId(),
            //client_email
            $order->getCustomerEmail(),
            //client_phone
            $order->getBillingAddress()->getTelephone(),
            //service_name
            '',
            //payment form url
            $this->getConfigData("form_url"), 
            //secret key
            $this->getConfigData("secret_key")
        );

        $cart_data = $order->getAllItems();
        foreach ($cart_data as $item) {
            if ($item->getProductType() != 'virtual' and $item->getProductType() != 'simple')
                continue;
            $taxes = array("tax" => "none", "tax_sum" => 0);
            $tax_rate = 0;
            $parent_item = $item->getParentItem();
            if (isset($parent_item)){
                $name = $parent_item->getName();
                $quantity = floatval($parent_item->getQtyOrdered());
                $price = $parent_item->getPrice()+($parent_item->getTaxAmount()/$quantity);
                $sum = $price*$quantity;
                $tax_rate = $parent_item->getTaxPercent(); 
            }
            else{
            $name = $item->getName();
            $quantity = floatval($item->getQtyOrdered());
            $price = $item->getPrice()+($item->getTaxAmount()/$quantity);
            $sum = $price*$quantity;
            $tax_rate = $item->getTaxPercent();
            }
            if ($tax_rate > 0) {
                $taxes = $this->paymentConfig->setTaxes($tax_rate);
            }
            $this->paymentConfig->updateFiscalCart($this->paymentConfig->getPaymentFormType(),
                        $name, $price, $quantity, $sum, $taxes["tax"]);
        }
        $this->paymentConfig->setShippingPrice($order->getShippingInclTax());
        if ($this->paymentConfig->getShippingPrice() > 0) {
            $shipping_taxes = array("tax" => "none", "tax_sum" => 0);
            if($order->getShippingTaxAmount() != 0){
                
                $shipping_tax_rate = ($order->getShippingTaxAmount()/$order->getShippingAmount())*100;
                $shipping_taxes = $this->paymentConfig->setTaxes($shipping_tax_rate); 
            }
            else {
                $shipping_taxes = $this->paymentConfig->setTaxes(0);
            }
             
            $shipping_name = $order->getShippingDescription();
            
            $this->paymentConfig->setUseDelivery(); //for precision correct check
            $this->paymentConfig->updateFiscalCart($this->paymentConfig->getPaymentFormType(), $shipping_name,
                            $this->paymentConfig->getShippingPrice(), 1, $this->paymentConfig->getShippingPrice(), $shipping_taxes["tax"]);
            $this->paymentConfig->delivery_index = count($this->paymentConfig->getFiscalCart())-1;
        }
        
        //set discounts
        $discount_exists = false;
        $discount_amount = $this->getOrder()->getDiscountAmount();
        if ($discount_amount < 0)
            $discount_exists = true;
        $this->paymentConfig->setDiscounts($discount_exists || $this->getConfigData("force_discount_check") == 1);

        //handle possible precision problem
        $this->paymentConfig->correctPrecision();

        $fiscal_cart_encoded = json_encode($this->paymentConfig->getFiscalCart());

        //generate payment form
        $to_hash = number_format($this->paymentConfig->getOrderTotal(), 2, ".", "") .
                   $this->paymentConfig->getOrderParams("clientid")     .
                   $this->paymentConfig->getOrderParams("orderid")      .
                   $this->paymentConfig->getOrderParams("service_name") .
                   $this->paymentConfig->getOrderParams("client_email") .
                   $this->paymentConfig->getOrderParams("client_phone") .
                   $this->paymentConfig->getOrderParams("secret_key");
        $sign = hash ('sha256' , $to_hash);
        return array ('form_url' =>         $this->paymentConfig->getOrderParams("form_url"),
                      'form_type' =>        $this->paymentConfig->getPaymentFormType(),
                      'params' => array(
                          "sum" =>          $this->paymentConfig->getOrderTotal(),
                          "clientid" =>     $this->paymentConfig->getOrderParams("clientid"),
                          "orderid" =>      $this->paymentConfig->getOrderParams("orderid"),
                          "client_email" => $this->paymentConfig->getOrderParams("client_email"),
                          "client_phone" => $this->paymentConfig->getOrderParams("client_phone"),
                          "phone" =>        $this->paymentConfig->getOrderParams("client_phone"),
                          "service_name" => $this->paymentConfig->getOrderParams("service_name"),
                          "cart" =>         $fiscal_cart_encoded,
                          "sign" =>         $sign,
        ));
    }
}
